<?php
  define( "INCLUDE_ROOT", $_SERVER['DOCUMENT_ROOT'] . "/includes/" );
  define( "CURRENT_ITEM", "Articles" );
  define( "CURRENT_ITEM_CLASS", " class=\"currentlistitem\"" );
  define( "CURRENT_ITEM_STYLE", " style=\"background-color: #C4C769\"" );
  
  // For everything that appears before the <head> tag
  include INCLUDE_ROOT . "pre_head.include";

  // The <head> tag is identical except for part of the <title> tag. The difference in the
  // title is what appears between this include and the following one.
  include INCLUDE_ROOT . "head_1.include";
  
  echo " Scripting FileMaker and iCal";
  
  include INCLUDE_ROOT . "head_2.include";
?>

<body>

<!-- The container ID encapsulates everything else on the page -->
<div id="container">

  <?php
    // The header css id is found entirely within this include.
    include INCLUDE_ROOT . "id_header.include";
  
    // The mainnav is the same for each page with one difference: each page belongs to
    // one of the mainnav links. The one it belongs to is colored differently, which is
    // why we have a separate include file for each section of the web site.
    include INCLUDE_ROOT . "id_mainnav.include";
  ?>
  <div id="column">
  <?php
    include INCLUDE_ROOT . "articles_first.include";
    include INCLUDE_ROOT . "filemaker_banners.include";
    include INCLUDE_ROOT . "latest_news.include";
  ?>  
  </div> <!-- Closing the column div -->
  
  <div id="contents">
    <div class="article">
      <h2>Scripting FileMaker and iCal</h2>

      <p class="note">This article was originally written by Chuck (with the assistance of Scott Love of The Moyer Group) for <a href="http://www.macworld.com">Macworld Magazine</a>, and appeared in the November 2003 issue. However, it doesn't appear to be available online anymore, so we're placing it here in the hopes that it will prove useful to some developers. The article was originally written in the days of FileMaker 6, but the concepts should be applicable to FileMaker 7.</p>

      <p>FileMaker Pro is easy to use and supremely scriptable via AppleScript; it lets you design databases that do almost anything, including interact with data in other programs. This sample project is a case in point. With it, you can tract a project's to-do items via a system created in FileMaker Pro (800/325-2747, <a href=http://www.filemaker.com">www.filemaker.com</a>). And it lets you push those to-do items to a handheld device via Apple's free <a href="http://www.apple.com/ical/">iCal</a> application and <a href="http://www.apple.com/isync/">iSync</a> utility. (For an AppleScript primer, see "Amazing AppleScript," December 2002.)</p>

      <h3>Needed: Two Databases</h3>
      
      <p>You can use FileMaker to create massive database systems, but for this example, we needed to create two small databases. (If you don't want to create these files or to type out the scripts, you can download them from <a href="http://www.macworld.com/downlods/fm-ical.sit">www.macworld.com/downloads/fm-ical.sit</a>.) Set up one database, named <span class="inline_code">ToDos.fp5</span>, that contains three Text fields (<span class="inline_code">Task</span>, <span class="inline_code">Calendar</span>, and <span class="inline_code">Priority</span>) and one Date field (<span class="inline_code">DueDate</span>). The other database, <span class="inline_code">Calendars.fp5</span>, should contain one Text field (<span class="inline_code">CalendarName</span>). In the <span class="inline_code">ToDos.fp5</span> database, format the <span class="inline_code">Priority</span> field as a pop-up list with an attached value list containing four options: <span class="inline_code">None</span>, <span class="inline_code">Very Important</span>, <span class="inline_code">Important</span>, and <span class="inline_code">Not Important</span>. You should also format the Calendar field as a pop-up list with a value list containing the contents of the field <span class="inline_code">CalendarName</span> in the <span class="inline_code">Calendars.fp5</span> file.</p>

      <h3>First Script: Sync Calendars</h3>
      
      <p>The first think you'll do is write a script that synchronizes your iCal calendars with the contents of the <span class="inline_code">Calendars.fp5</span> database; if you create new calendars within FileMaker, they'll be added to iCal, and if you add new calendars to iCal, FileMaker will pick them up automatically.</p>

      <p>You use ScriptMaker, FileMaker's built-in automation system, to embed an AppleScript within FileMaker. While <span class="inline_code">ToDos.fp5</span> is active, select <span class="os_font">Scripts>ScriptMaker</span>. Type a new script name, <span class="inline_code">iCal Sync</span>, and click on <span class="os_font">Create</span>. FileMaker will pen a script-definition dialog box with a pane to the left with your script choices. (Delete the items it has placed in the right pane by default.) Since your script requires that the <span class="inline_code">Calendars.fp5</span> file also be open, scroll to near the bottom of the <span class="inline_code">Files</span> subsection and double-click on <span class="inline_code">Open</span>. Then click on the <span class="os_font">Specify File</span> button and choose your <span class="inline_code">Calendars.fp5</span> file.</p>

      <p>To embed any AppleScript code in FileMaker, you need to scroll almost to the bottom of the left pane and double-click on <span class="inline_code">Perform AppleScript</span>; then click on the <span class="os_font">Specify</span> button. In the resulting window, you can type or paste your AppleScript code:</p>

<code>set iCalList to {}<br />
tell application "iCal"<br />
&nbsp;&nbsp;repeat with theCal in every calendar<br />
&nbsp;&nbsp;&nbsp;&nbsp;set iCalList to iCalList & {title of theCal}<br />
&nbsp;&nbsp;end repeat<br />
end tell<br />
set FMCalList to every record of database "Calendars.fp5"<br />
repeat with theCalendar in iCalList<br />
&nbsp;&nbsp;if theCalendar is not in FMCalList then &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;create new record at database &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"Calendars.fp5" with data theCalendar<br />
end repeat<br />
repeat with theCalendar in FMCalList<br />
&nbsp;&nbsp;if theCalendar is not in iCalList then &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;tell application "iCal" to &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;make new calendar with properties &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{writable:true} & &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{title:theCalendar}<br />
end repeat</code>

      <p>The script stores the available calendar names from iCal and FileMaker is a few list variables (<span class="inline_code">iCalList</span> and <span class="inline_code">FMCalList</span>), and then loops through each list and compares the calendars in FileMaker and iCal. If calendar names that appear in one list are missing in the other, the script supplies the missing data.</p>

      <p>While this script explicitly targets the iCal application, it doesn't target FileMaker by name. Because the script is stored within FileMaker itself, the program assumes that any commands not targeted elsewhere are meant for it.</p>

      <p>To make this script run every time your database file opens, select <span class="os_font">FileMaker Pro>Preferences>Document</span>. In the <span class="os_font">When Opening 'ToDos.fp5'</span> section, select the <span class="os_font">Perform Script</span> option and then select <span class=os_font">iCal Sync</span>.</p>

      <p>When creating the new record in FileMaker, the script uses the <span class="inline_code">with data</span> qualifier for <span class="inline_code">create new record</span>. In this case, your record has only one field, so the data is a simple string (stored in the <span class="inline_code">theCalendar</span> variable). However, if the database contained multiple fields, you could accomplish the same type of operation by using a list: the data's position in the list would determine which field received the data, with the creation order of the fields in FileMaker setting the order in which AppleScript populates them. You can also set individual fields (called <em>cells</em> in FileMaker terminology) in a record by using a command such as <span class="inline_code">set cell "CalendarName" of current record of database "Calendars.fp5" to theCalendar</span>.</p>

      <h3>Second Script: Export Events</h3>
      
      <p>The previous script used iCal data to add items to FileMaker. Now you can create a new iCal To-Do item from a FileMaker record. (Use the method described earlier to create a new ScriptMaker script &mdash; this one called <span class="inline_code">Create ToDo</span> &mdash; in the <span class="inline_code">ToDos.fp5</span> database, with a <span class="inline_code">Perform AppleScript</span> item within that script.)</p>

<code>tell current reocrd of database "ToDos.fp5"<br />
&nbsp;&nbsp;set theDueDate to cell "DueDate"<br />
&nbsp;&nbsp;set theSummary to cell "Task"<br />
&nbsp;&nbsp;set thePriorityName to cell "Priority"<br />
&nbsp;&nbsp;set theCalendar to cell "Calendar"<br />
end tell<br />
set theDueDate to date theDueDate<br />
set thePriority to 0<br />
set prioritiesList to &#172<br />
&nbsp;&nbsp;{{priorityName:"None", priorityNumber:0}} & &#172<br />
&nbsp;&nbsp;{{priorityName:"Very Important", priorityNumber:1}} & &#172<br />
&nbsp;&nbsp;{{priorityName:"Important", priorityNumber:5}} & &#172<br />
&nbsp;&nbsp;{{priorityName:"Not Important", priorityNumber:9}}<br />
repeat with currentPriority in prioritiesList<br />
&nbsp;&nbsp;if thePriority is priorityName of currentPriority then<br />
&nbsp;&nbsp;&nbsp;&nbsp;set thePriority to priorityNumber of currentPriority<br />
&nbsp;&nbsp;&nbsp;&nbsp;exit repeat<br />
&nbsp;&nbsp;end if<br />
end repeat<br />
tell application "iCal"<br />
&nbsp;&nbsp;set theCal to ""<br />
&nbsp;&nbsp;repeat with i from 1 to count of every calendar<br />
&nbsp;&nbsp;&nbsp;&nbsp;if title of (item i of every calendar) is theCalendar then &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;set theCal to item i of every calendar<br />
&nbsp;&nbsp;end repeat<br />
&nbsp;&nbsp;if theCal is "" then<br />
&nbsp;&nbsp;&nbsp;&nbsp;set theCal to make new calendar with properties &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{writable:true} & &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{title:theCalendar}<br />
&nbsp;&nbsp;&nbsp;&nbsp;tell application "FileMaker Pro to &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;create new record at database &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"Calendars.fp5" with data theCalendar<br />
&nbsp;&nbsp;end if<br />
&nbsp;&nbsp;make todo at end of todos of theCal with properties &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;{due date:theDueDate} & &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;{summary:theSummary} & &#172<br />
&nbsp;&nbsp;&nbsp;&nbsp;{priority:thePriority}<br />
end tell</code>

      <p>This script extracts the data from the current FileMaker field and stores each item in a variable (<span class="inline_code">theDueDate</span>, <span class="inline_code">theSummary</span>, and so on). Note that we saved a good deal of typing by using the <span class="inline_code">tell</span> block that b egins with <span class="inline_code">tell current record of database "ToDos.fp5"</span>, rather than repeating the same information in all four lines.</p>

      <p>The line <span class="inline_code">set theDueDate to date theDueDate</span> may seem repetitive, but it's necessary. FileMaker returns the contents of a Date field as a text string. This line converts that text into a proper AppleScript date object.</p>

      <p>Next, the script instructions create the variable <span class="inline_code">prioritiesList</span>, which maps a list of priority names to their iCal-equivalent values. For example, <span class="inline_code">Very Important</span> gets converted to an iCal priority level of <span class="inline_code">1</span>.</p>

      <p>Finally, the script tells iCal what to do. Since your database allows the user to choose a calendar for each to-do item, the subsequent <span class="inline_code">repeat</span> loop matches the selection to the calendar in iCal and sets that calendar in the variable <span class="inline_code">theCal</span>. If you entered a calendar that isn't already in iCal (and you haven't run the <span class="inline_code">iCal Sync</span> in the meantime), the initial value of theCal will be empty &mdash; in AppleScript terms, <span class="inline_code">""</span>. If that's the case, the script creates a new calendar in both iCal and in <span class="inline_code">Calendars.fp5</span>, storing a reference to the iCal calendar within the <span class="inline_code">theCal</span> variable.</p>

      <p>Finally, with the code beginning <span class="inline_code">make todo at end of todos</span>, we tell iCal to add the FileMaker to-do item to iCal.</p>

      <h3>The Final Touch: Add a Button in FileMaker</h3>
      
      <p>You can run your script from FileMaker's <span class="os_font">ScriptMaker</span> menu, but to make it even easier to use, wire it to a FileMaker button. In <span class="os_font">Layout</span> mode, draw a button with the button tool. When the <span class="os_font">Specify Button</span> dialog box appears, click on the <span class="os_font">Perform Script</span> step in the list on the left, and select <span class=inline_code">Create ToDo</span> from the <span class="os_font">Specify</span> pop-up menu on the right.</p>

      <p>Back in <span class="os_font">Browse</span> mode, try entering some data in the record fields, and then click on the button. Switch to iCal, and you should see your new to-do. Now you're free to take your project to-dos on the road, secure in the knowledge that they agree with your FileMaker data at the office.</p>
      
    </div> <!-- Closing the article div -->
  </div> <!-- Closing the contents div -->

  <?php // Just the copyright information so far.
     include INCLUDE_ROOT . "id_footer.include"; ?>

</div> <!-- Closing the contents div -->
</body>
</html>
